//+------------------------------------------------------------------+
//|                                      PositionPropertiesPanel.mq5 |
//|                        Copyright 2012, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//--- Include a class of the Standard Library
#include <Trade/Trade.mqh>
//---
#define INFOPANEL_SIZE 14 // Size of the array for info panel objects
#define EXPERT_NAME MQL5InfoString(MQL5_PROGRAM_NAME) // Name of the Expert Advisor
//--- GLOBAL VARIABLES
bool                 pos_open=false;         // Flag of presence/absence of an open position
string               pos_symbol="";          // Symbol
long                 pos_magic=0;            // Magic number
string               pos_comment="";         // Comment
double               pos_swap=0.0;           // Swap
double               pos_commission=0.0;     // Commission
double               pos_price=0.0;          // Position price
double               pos_cprice=0.0;         // Current price of the position
double               pos_profit=0.0;         // Profit/Loss of the position
double               pos_volume=0.0;         // Position volume
double               pos_sl=0.0;             // Stop Loss of the position
double               pos_tp=0.0;             // Take Profit of the position
datetime             pos_time=NULL;          // Position opening time
long                 pos_id=0;               // Position identifier
ENUM_POSITION_TYPE   pos_type=WRONG_VALUE;   // Position type
//--- Array of names of objects that display the names of position properties
string pos_prop_names[INFOPANEL_SIZE]=
  {
   "name_pos_symbol",
   "name_pos_magic",
   "name_pos_comment",
   "name_pos_swap",
   "name_pos_commission",
   "name_pos_price",
   "name_pos_cprice",
   "name_pos_profit",
   "name_pos_volume",
   "name_pos_sl",
   "name_pos_tp",
   "name_pos_time",
   "name_pos_id",
   "name_pos_type"
  };
//---
// Array of names of objects that display values of position properties
string pos_prop_values[INFOPANEL_SIZE]=
  {
   "value_pos_symbol",
   "value_pos_magic",
   "value_pos_comment",
   "value_pos_swap",
   "value_pos_commission",
   "value_pos_price",
   "value_pos_cprice",
   "value_pos_profit",
   "value_pos_volume",
   "value_pos_sl",
   "value_pos_tp",
   "value_pos_time",
   "value_pos_id",
   "value_pos_type"
  };
//--- Array of position property names
string pos_prop_texts[INFOPANEL_SIZE]=
  {
   "Symbol :",
   "Magic Number :",
   "Comment :",
   "Swap :",
   "Commission :",
   "Open Price :",
   "Current Price :",
   "Profit :",
   "Volume :",
   "Stop Loss :",
   "Take Profit :",
   "Time :",
   "Identifier :",
   "Type :"
  };
//--- Price data arrays
double               close_price[]; // Close (closing prices of the bar)
double               open_price[];  // Open (opening prices of the bar)
//--- Load the class
CTrade trade;

//+------------------------------------------------------------------+
//| DEFINING VARIABLES AND BOLLINGER BANDS READING FUNCTION          |
//+------------------------------------------------------------------+

input bool            UseParentChartPeriod=true;      // Use Parent Chart Period for Bollinger Bands

input ENUM_TIMEFRAMES ParentChartPeriod=PERIOD_M10;   // Parent Chart Period
input ENUM_TIMEFRAMES ChildChartPeriod =PERIOD_M1;    // Child Chart Period

struct                BollingerStruct
                       {double      UpperValue;
                        double     MiddleValue;
                        double      LowerValue;};

int               MaxBars=1000;
BollingerStruct   Bollinger;
double            UpperBuffer[];
double            MiddleBuffer[];
double            LowerBuffer[]; 

bool ReadBollinger(string CustomSymbol, ENUM_TIMEFRAMES CustomPeriod) {
  string Message;
  int                  Cols=8,Rows=3;
  int                  bands_period=20;           // period of moving average
  int                  bands_shift=0;             // shift
  double               deviation=2.0;             // number of standard deviations
  ENUM_APPLIED_PRICE   applied_price=PRICE_CLOSE; // type of price
  int InpInd_Handle=iBands(CustomSymbol,CustomPeriod,bands_period,bands_shift,deviation,applied_price);
  if(InpInd_Handle==INVALID_HANDLE) Print(" Error: Cannot load Bollinger");
  if(CopyBuffer(InpInd_Handle,0,0,MaxBars,MiddleBuffer)<0)
     {PrintFormat("Failed to copy data from the iBands indicator, error code %d",GetLastError()); return(false);}
  if(CopyBuffer(InpInd_Handle,1,0,MaxBars,UpperBuffer)<0)
     {PrintFormat("Failed to copy data from the iBands indicator, error code %d",GetLastError()); return(false);}
  if(CopyBuffer(InpInd_Handle,2,0,MaxBars,LowerBuffer)<0)
     {PrintFormat("Failed to copy data from the iBands indicator, error code %d",GetLastError()); return(false);}
   Bollinger.UpperValue=UpperBuffer[MaxBars-1];
   Bollinger.MiddleValue=MiddleBuffer[MaxBars-1];
   Bollinger.LowerValue=LowerBuffer[MaxBars-1];
   return(true);}

//+------------------------------------------------------------------+
//| CHECKING FOR THE NEW BAR                                         |
//+------------------------------------------------------------------+
bool CheckNewBar()
  {
//--- Variable for storing the opening time of the current bar
   static datetime new_bar=NULL;
//--- Array for getting the opening time of the current bar
   static datetime time_last_bar[1]={0};
//--- Get the opening time of the current bar
//    If an error occurred when getting the time, print the relevant message
   if(CopyTime(_Symbol,Period(),0,1,time_last_bar)==-1)
     { Print(__FUNCTION__,": Error copying the opening time of the bar: "+IntegerToString(GetLastError())+""); }
//--- If this is a first function call
   if(new_bar==NULL)
     {
      // Set the time
      new_bar=time_last_bar[0];
      Print(__FUNCTION__,": Initialization ["+_Symbol+"][TF: "+TimeframeToString(Period())+"]["
            +TimeToString(time_last_bar[0],TIME_DATE|TIME_MINUTES|TIME_SECONDS)+"]");
      return(false); // Return false and exit 
     }
//--- If the time is different
   if(new_bar!=time_last_bar[0])
     {
      new_bar=time_last_bar[0]; // Set the time and exit 
      return(true); // Store the time and return true
     }
//--- If we have reached this line, then the bar is not new, return false
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string TimeframeToString(ENUM_TIMEFRAMES timeframe)
  {
   string str="";
//--- If the passed value is incorrect, take the time frame of the current chart
   if(timeframe==WRONG_VALUE|| timeframe== NULL)
      timeframe= Period();
   switch(timeframe)
     {
      case PERIOD_M1  : str="M1";  break;
      case PERIOD_M2  : str="M2";  break;
      case PERIOD_M3  : str="M3";  break;
      case PERIOD_M4  : str="M4";  break;
      case PERIOD_M5  : str="M5";  break;
      case PERIOD_M6  : str="M6";  break;
      case PERIOD_M10 : str="M10"; break;
      case PERIOD_M12 : str="M12"; break;
      case PERIOD_M15 : str="M15"; break;
      case PERIOD_M20 : str="M20"; break;
      case PERIOD_M30 : str="M30"; break;
      case PERIOD_H1  : str="H1";  break;
      case PERIOD_H2  : str="H2";  break;
      case PERIOD_H3  : str="H3";  break;
      case PERIOD_H4  : str="H4";  break;
      case PERIOD_H6  : str="H6";  break;
      case PERIOD_H8  : str="H8";  break;
      case PERIOD_H12 : str="H12"; break;
      case PERIOD_D1  : str="D1";  break;
      case PERIOD_W1  : str="W1";  break;
      case PERIOD_MN1 : str="MN1"; break;
     }
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| GETTING BAR VALUES                                               |
//+------------------------------------------------------------------+
void GetBarsData()
  {
//--- Number of bars for getting their data in an array
   int amount=2;
//--- Reverse the time series ... 3 2 1 0
   ArraySetAsSeries(close_price,true);
   ArraySetAsSeries(open_price,true);
//--- Get the closing price of the bar
//    If the number of the obtained values is less than requested, print the relevant message
   if(CopyClose(_Symbol,Period(),0,amount,close_price)<amount)
     {
      Print("Failed to copy the values ("
            +_Symbol+", "+TimeframeToString(Period())+") to the Close price array! "
            "Error "+IntegerToString(GetLastError())+": "+ErrorDescription(GetLastError()));
     }
//--- Get the opening price of the bar
//    If the number of the obtained values is less than requested, print the relevant message
   if(CopyOpen(_Symbol,Period(),0,amount,open_price)<amount)
     {
      Print("Failed to copy the values ("
            +_Symbol+", "+TimeframeToString(Period())+") to the Open price array! "
            "Error "+IntegerToString(GetLastError())+": "+ErrorDescription(GetLastError()));
     }
  }
//+------------------------------------------------------------------+
//| DETERMINING TRADING SIGNALS                                      |
//+------------------------------------------------------------------+
int GetTradingSignal()
  {
//--- A Buy signal (0) :
   if(close_price[1]>open_price[1])
      return(0);
//--- A Sell signal (1) :
   if(close_price[1]<open_price[1])
      return(1);
//--- No signal (3):
   return(3);
  }
//+------------------------------------------------------------------+
//| TRADING BLOCK                                                    |
//+------------------------------------------------------------------+
void TradingBlock()
  {
   int               signal=-1;           // Variable for getting a signal
   string            comment="hello :)";  // Position comment
   double            start_lot=0.1;       // Initial volume of a position
   double            lot=0.0;             // Volume for position calculation in case of reverse position
   double            ask=0.0;             // Ask price
   double            bid=0.0;             // Bid price
   
//+------------------------------------------------------------------+
//| CHECKING ABILITY TO READ BOLLINGER                               |
//+------------------------------------------------------------------+  

   if(UseParentChartPeriod)
     ReadBollinger(_Symbol,ParentChartPeriod);
   else
     ReadBollinger(_Symbol,ChildChartPeriod);

//--- Get a signal
   signal=GetTradingSignal();
//--- Find out if there is a position
   pos_open=PositionSelect(_Symbol);
//--- If it is a Buy signal
   if(signal==0)
     {
      //--- Get the Ask price
      ask=NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_ASK),_Digits);
      //--- If there is no position
      if(!pos_open)
        {
         //--- Open a position
         //    If the position failed to open, print the relevant message
         if(!trade.PositionOpen(_Symbol,ORDER_TYPE_BUY,lot,ask,0,0,comment))
           {
            Print("Error opening a BUY position: ",GetLastError(),
                  " - ",ErrorDescription(GetLastError()));
           }
        }
      //--- If there is a position
      else
        {
         //--- Get the position type
         pos_type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
         //--- If it is a SELL position
         if(pos_type==POSITION_TYPE_SELL)
           {
            //--- Get the position volume
            pos_volume=PositionGetDouble(POSITION_VOLUME);
            //--- Adjust the volume
            lot=NormalizeDouble(pos_volume+start_lot,2);
            //--- Open a position. If the position failed to open, print the relevant message
            if(!trade.PositionOpen(_Symbol,ORDER_TYPE_BUY,lot,ask,0,0,comment))
              {
               Print("Error opening a SELL position: ",GetLastError(),
                     " - ",ErrorDescription(GetLastError()));
              }
           }
        }
      //---
      return;
     }
//--- If there is a Sell signal
   if(signal==1)
     {
      //-- Get the Bid price
      bid=NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_BID),_Digits);
      //--- If there is no position
      if(!pos_open)
        {
         //--- Open a position. If the position failed to open, print the relevant message
         if(!trade.PositionOpen(_Symbol,ORDER_TYPE_SELL,start_lot,bid,0,0,comment))
           { Print("Error opening a SELL position: ",GetLastError()," - ",ErrorDescription(GetLastError())); }
        }
      //--- If there is a position
      else
        {
         //--- Get the position type
         pos_type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
         //--- If it is a BUY position
         if(pos_type==POSITION_TYPE_BUY)
           {
            //--- Get the position volume
            pos_volume=PositionGetDouble(POSITION_VOLUME);
            //--- Adjust the volume
            lot=NormalizeDouble(pos_volume+start_lot,2);
            //--- Open a position. If the position failed to open, print the relevant message
            if(!trade.PositionOpen(_Symbol,ORDER_TYPE_SELL,lot,bid,0,0,comment))
              { Print("Error opening a SELL position: ",GetLastError()," - ",ErrorDescription(GetLastError())); }
           }
        }
      //---
      return;
     }
  }
//+------------------------------------------------------------------+
//| CREATING THE EDIT OBJECT                                         |
//+------------------------------------------------------------------+
void CreateEdit(long             chart_id,         // chart id
                int              sub_window,       // (sub)window number
                string           name,             // object name
                string           text,             // displayed text
                ENUM_BASE_CORNER corner,           // chart corner
                string           font_name,        // font
                int              font_size,        // font size
                color            font_color,       // font color
                int              x_size,           // width
                int              y_size,           // height
                int              x_distance,       // X-coordinate
                int              y_distance,       // Y-coordinate
                long             z_order,          // Z-order
                color            background_color, // background color
                bool             read_only)        // Read Only flag
  {
// If the object has been created successfully...
   if(ObjectCreate(chart_id,name,OBJ_EDIT,sub_window,0,0))
     {
      // ...set its properties
      ObjectSetString(chart_id,name,OBJPROP_TEXT,text);                 // displayed text
      ObjectSetInteger(chart_id,name,OBJPROP_CORNER,corner);            // set the chart corner
      ObjectSetString(chart_id,name,OBJPROP_FONT,font_name);            // set the font
      ObjectSetInteger(chart_id,name,OBJPROP_FONTSIZE,font_size);       // set the font size
      ObjectSetInteger(chart_id,name,OBJPROP_COLOR,font_color);         // font color
      ObjectSetInteger(chart_id,name,OBJPROP_BGCOLOR,background_color); // background color
      ObjectSetInteger(chart_id,name,OBJPROP_XSIZE,x_size);             // width
      ObjectSetInteger(chart_id,name,OBJPROP_YSIZE,y_size);             // height
      ObjectSetInteger(chart_id,name,OBJPROP_XDISTANCE,x_distance);     // set the X-coordinate
      ObjectSetInteger(chart_id,name,OBJPROP_YDISTANCE,y_distance);     // set the Y-coordinate
      ObjectSetInteger(chart_id,name,OBJPROP_SELECTABLE,false);         // cannot select the object if FALSE
      ObjectSetInteger(chart_id,name,OBJPROP_ZORDER,z_order);           // Z-order of the object
      ObjectSetInteger(chart_id,name,OBJPROP_READONLY,read_only);       // Read Only
      ObjectSetInteger(chart_id,name,OBJPROP_ALIGN,ALIGN_LEFT);         // align left
      ObjectSetString(chart_id,name,OBJPROP_TOOLTIP,"\n");              // no tooltip if "\n"
     }
  }
//+------------------------------------------------------------------+
//| CREATING THE LABEL OBJECT                                        |
//+------------------------------------------------------------------+
void CreateLabel(long               chart_id,   // chart id
                 int                sub_window, // (sub)window number
                 string             name,       // object name
                 string             text,       // displayed text
                 ENUM_ANCHOR_POINT  anchor,     // anchor point
                 ENUM_BASE_CORNER   corner,     // chart corner
                 string             font_name,  // font
                 int                font_size,  // font size
                 color              font_color, // font color
                 int                x_distance, // X-coordinate
                 int                y_distance, // Y-coordinate
                 long               z_order)    // Z-order
  {
// If the object has been created successfully...
   if(ObjectCreate(chart_id,name,OBJ_LABEL,sub_window,0,0))
     {
      // ...set its properties
      ObjectSetString(chart_id,name,OBJPROP_TEXT,text);              // displayed text
      ObjectSetString(chart_id,name,OBJPROP_FONT,font_name);         // set the font
      ObjectSetInteger(chart_id,name,OBJPROP_COLOR,font_color);      // set the font color
      ObjectSetInteger(chart_id,name,OBJPROP_ANCHOR,anchor);         // set the anchor point
      ObjectSetInteger(chart_id,name,OBJPROP_CORNER,corner);         // set the chart corner
      ObjectSetInteger(chart_id,name,OBJPROP_FONTSIZE,font_size);    // set the font size
      ObjectSetInteger(chart_id,name,OBJPROP_XDISTANCE,x_distance);  // set the X-coordinate
      ObjectSetInteger(chart_id,name,OBJPROP_YDISTANCE,y_distance);  // set the Y-coordinate
      ObjectSetInteger(chart_id,name,OBJPROP_SELECTABLE,false);      // cannot select the object if FALSE
      ObjectSetInteger(chart_id,name,OBJPROP_ZORDER,z_order);        // Z-order of the object
      ObjectSetString(chart_id,name,OBJPROP_TOOLTIP,"\n");           // no tooltip if "\n"
     }
  }
//+------------------------------------------------------------------+
//| DELETING THE OBJECT BY NAME                                      |
//+------------------------------------------------------------------+
void DeleteObjectByName(string name)
  {
   int  sub_window=0;      // Returns the number of the subwindow where the object is located
   bool res       =false;  // Result following an attempt to delete the object
//--- Find the object by name
   sub_window=ObjectFind(ChartID(),name);
//---
   if(sub_window>=0) // If it has been found,..
     {
      res=ObjectDelete(ChartID(),name); // ...delete it
      //---
      // If an error occurred when deleting the object, print the relevant message
      if(!res)
         Print("Error deleting the object: ("+IntegerToString(GetLastError())+"): "+ErrorDescription(GetLastError()));
     }
  }
//+------------------------------------------------------------------+
//| ZEROING OUT VARIABLES FOR POSITION PROPERTIES                    |
//+------------------------------------------------------------------+
void ZeroPositionProperties()
  {
   pos_symbol     ="";
   pos_comment    ="";
   pos_magic      =0;
   pos_price      =0.0;
   pos_cprice     =0.0;
   pos_sl         =0.0;
   pos_tp         =0.0;
   pos_type       =WRONG_VALUE;
   pos_volume     =0.0;
   pos_commission =0.0;
   pos_swap       =0.0;
   pos_profit     =0.0;
   pos_time       =NULL;
   pos_id         =0;
  }
//+------------------------------------------------------------------+
//| CONVERTING POSITION TYPE TO A STRING                             |
//+------------------------------------------------------------------+
string PositionTypeToString(ENUM_POSITION_TYPE type)
  {
   string str="";
//---
   if(type==POSITION_TYPE_BUY)
      str="buy";
   else if(type==POSITION_TYPE_SELL)
      str="sell";
   else
      str="wrong value";
//---
   return(str);
  }
//+------------------------------------------------------------------+
//| GETTING POSITION PROPERTIES                                      |
//+------------------------------------------------------------------+
void GetPositionProperties()
  {
// Check if there is an open position
   pos_open=PositionSelect(_Symbol);
//---
   if(pos_open) // If an open position exists, get its properties
     {
      pos_symbol     =PositionGetString(POSITION_SYMBOL);
      pos_comment    =PositionGetString(POSITION_COMMENT);
      pos_magic      =PositionGetInteger(POSITION_MAGIC);
      pos_price      =PositionGetDouble(POSITION_PRICE_OPEN);
      pos_cprice     =PositionGetDouble(POSITION_PRICE_CURRENT);
      pos_sl         =PositionGetDouble(POSITION_SL);
      pos_tp         =PositionGetDouble(POSITION_TP);
      pos_type       =(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
      pos_volume     =PositionGetDouble(POSITION_VOLUME);
      pos_commission =PositionGetDouble(POSITION_COMMISSION);
      pos_swap       =PositionGetDouble(POSITION_SWAP);
      pos_profit     =PositionGetDouble(POSITION_PROFIT);
      pos_time       =(datetime)PositionGetInteger(POSITION_TIME);
      pos_id         =PositionGetInteger(POSITION_IDENTIFIER);
     }
   else // If there is no open position, zero out variables for position properties
   ZeroPositionProperties();
//---
   SetInfoPanel(); // Set/update the info panel
  }
//+------------------------------------------------------------------+
//| SETTING THE INFO PANEL                                           |
//|------------------------------------------------------------------+
void SetInfoPanel()
  {
   int               y_bg=18;             // Y-coordinate for the background and header
   int               y_property=32;       // Y-coordinate for the list of properties and their values
   int               line_height=12;      // Line height
//---
   int               font_size=8;         // Font size
   string            font_name="Calibri"; // Font
   color             font_color=clrWhite; // Font color
//---
   ENUM_ANCHOR_POINT anchor=ANCHOR_RIGHT_UPPER; // Anchor point in the top right corner
   ENUM_BASE_CORNER  corner=CORNER_RIGHT_UPPER; // Origin of coordinates in the top right corner of the chart
//--- X-coordinates
   int               x_first_column=120;  // First column (names of properties)
   int               x_second_column=10;  // Second column (values of properties)
//--- Testing in the visualization mode
   if(MQL5InfoInteger(MQL5_VISUAL_MODE))
     {
      y_bg=2;
      y_property=16;
     }
//--- Array of Y-coordinates for the names of position properties and their values
   int               y_prop_array[INFOPANEL_SIZE]={0};
//--- Fill the array with coordinates for each line on the info panel
   for(int i=0; i<INFOPANEL_SIZE; i++)
     {
      if(i==0) y_prop_array[i]=y_property;
      else     y_prop_array[i]=y_property+line_height*i;
     }
//--- Background of the info panel
   CreateEdit(0,0,"InfoPanelBackground","",corner,font_name,8,clrWhite,230,190,231,y_bg,0,C'15,15,15',true);
//--- Header of the info panel
   CreateEdit(0,0,"InfoPanelHeader","  POSITION  PROPERTIES",corner,font_name,8,clrWhite,230,14,231,y_bg,1,clrFireBrick,true);
//--- List of the names of position properties and their values
   for(int i=0; i<INFOPANEL_SIZE; i++)
     {
      //--- Property name
      CreateLabel(0,0,pos_prop_names[i],pos_prop_texts[i],anchor,corner,font_name,font_size,font_color,x_first_column,y_prop_array[i],2);
      //--- Property value
      CreateLabel(0,0,pos_prop_values[i],GetPropertyValue(i),anchor,corner,font_name,font_size,font_color,x_second_column,y_prop_array[i],2);
     }
//---
   ChartRedraw(); // Redraw the chart
  }
//+------------------------------------------------------------------+
//| DELETING THE INFO PANEL                                          |
//+------------------------------------------------------------------+
void DeleteInfoPanel()
  {
   DeleteObjectByName("InfoPanelBackground");   // Delete the panel background
   DeleteObjectByName("InfoPanelHeader");       // Delete the panel header
//--- Delete position properties and their values
   for(int i=0; i<INFOPANEL_SIZE; i++)
     {
      DeleteObjectByName(pos_prop_names[i]);    // Delete the property
      DeleteObjectByName(pos_prop_values[i]);   // Delete the value
     }
//---
   ChartRedraw(); // Redraw the chart
  }
//+------------------------------------------------------------------+
//| RETURNING THE STRING WITH POSITION PROPERTY VALUE                |
//+------------------------------------------------------------------+
string GetPropertyValue(int number)
  {
//--- Sign indicating the lack of an open position or a certain property
//    E.g. the lack of a comment, Stop Loss or Take Profit
   string empty="-";
//--- If an open position exists, return the value of the requested property
   if(pos_open)
     {
      switch(number)
        {
         case 0  : return(pos_symbol);                                           break;
         case 1  : return(IntegerToString((int)pos_magic));                      break;
         //--- return the value of the comment, if any, otherwise return the sign indicating the lack of comment
         case 2  : return(pos_comment!="" ? pos_comment : empty);                break;
         case 3  : return(DoubleToString(pos_swap,2));                           break;
         case 4  : return(DoubleToString(pos_commission,2));                     break;
         case 5  : return(DoubleToString(pos_price,_Digits));                    break;
         case 6  : return(DoubleToString(pos_cprice,_Digits));                   break;
         case 7  : return(DoubleToString(pos_profit,2));                         break;
         case 8  : return(DoubleToString(pos_volume,2));                         break;
         case 9  : return(pos_sl!=0.0 ? DoubleToString(pos_sl,_Digits) : empty); break;
         case 10 : return(pos_tp!=0.0 ? DoubleToString(pos_tp,_Digits) : empty); break;
         case 11 : return(TimeToString(pos_time,TIME_DATE|TIME_MINUTES));        break;
         case 12 : return(IntegerToString((int)pos_id));                         break;
         case 13 : return(PositionTypeToString(pos_type));                       break;

         default : return(empty);
        }
     }
//---
// If there is no open position, return the sign indicating the lack of the open position "-"
   return(empty);
  }
//+------------------------------------------------------------------+
//| RETURNING A TEXTUAL DESCRIPTION OF THE DEINITIALIZATION REASON CODE|
//+------------------------------------------------------------------+
string GetDeinitReasonText(int reason_code)
  {
   string text="";
//---
   switch(reason_code)
     {
      case REASON_PROGRAM :     // 0
         text="The Expert Advisor has stopped working calling the ExpertRemove() function.";    break;
      case REASON_REMOVE :      // 1
         text="The '"+EXPERT_NAME+"' program has been removed from the chart.";                 break;
      case REASON_RECOMPILE :   // 2
         text="The '"+EXPERT_NAME+"' program has been recompiled.";                             break;
      case REASON_CHARTCHANGE : // 3
         text="Chart symbol or period has been changed.";                                       break;
      case REASON_CHARTCLOSE :  // 4
         text="The chart is closed.";                                                           break;
      case REASON_PARAMETERS :  // 5
         text="Input parameters have been changed by the user.";                                break;
      case REASON_ACCOUNT :     // 6
         text="A different account has been activated.";                                        break;
      case REASON_TEMPLATE :    // 7
         text="A different chart template has been applied.";                                   break;
      case REASON_INITFAILED :  // 8
         text="A flag specifying that the OnInit() handler returned zero value.";               break;
      case REASON_CLOSE :       // 9
         text="The terminal has been closed.";                                                  break;
      default : text="The reason is undefined.";
     }
//---
   return text;
  }
//+------------------------------------------------------------------+
//| RETURNING THE ERROR DESCRIPTION                                  |
//+------------------------------------------------------------------+
string ErrorDescription(int error_code)
  {
   string error_string="";
//---
   switch(error_code)
     {
      //--- Trade server return codes

      case 10004: error_string="Requote";                                                                                                                               break;
      case 10006: error_string="Request rejected";                                                                                                                      break;
      case 10007: error_string="Request canceled by trader";                                                                                                            break;
      case 10008: error_string="Order placed";                                                                                                                          break;
      case 10009: error_string="Request executed";                                                                                                                      break;
      case 10010: error_string="Request executed partially";                                                                                                            break;
      case 10011: error_string="Request processing error";                                                                                                              break;
      case 10012: error_string="Request timed out";                                                                                                                     break;
      case 10013: error_string="Invalid request";                                                                                                                       break;
      case 10014: error_string="Invalid request volume";                                                                                                                break;
      case 10015: error_string="Invalid request price";                                                                                                                 break;
      case 10016: error_string="Invalid Stop orders in the request";                                                                                                    break;
      case 10017: error_string="Trading forbidden";                                                                                                                     break;
      case 10018: error_string="Market is closed";                                                                                                                      break;
      case 10019: error_string="Insufficient funds";                                                                                                                    break;
      case 10020: error_string="Prices changed";                                                                                                                        break;
      case 10021: error_string="No quotes to process the request";                                                                                                      break;
      case 10022: error_string="Invalid order expiration in the request";                                                                                               break;
      case 10023: error_string="Order status changed";                                                                                                                  break;
      case 10024: error_string="Too many requests";                                                                                                                     break;
      case 10025: error_string="No changes in the request";                                                                                                             break;
      case 10026: error_string="Automated trading is disabled by trader";                                                                                               break;
      case 10027: error_string="Automated trading is disabled by the client terminal";                                                                                  break;
      case 10028: error_string="Request blocked for processing";                                                                                                        break;
      case 10029: error_string="Order or position frozen";                                                                                                              break;
      case 10030: error_string="The specified type of order execution by balance is not supported";                                                                     break;
      case 10031: error_string="No connection with trade server";                                                                                                       break;
      case 10032: error_string="Transaction is allowed for live accounts only";                                                                                         break;
      case 10033: error_string="You have reached the maximum number of pending orders";                                                                                 break;
      case 10034: error_string="You have reached the maximum order and position volume for this symbol";                                                                break;

      //--- Runtime errors

      case 0:  // The operation performed successfully
      case 4001: error_string="Unexpected internal error";                                                                                                              break;
      case 4002: error_string="Incorrect parameter in the internal call of the client terminal function";                                                               break;
      case 4003: error_string="Incorrect parameter in the call of the system function";                                                                                 break;
      case 4004: error_string="Not enough memory to perform the system function";                                                                                       break;
      case 4005: error_string="The structure contains string and/or dynamic array objects and/or structures with such objects and/or classes";                          break;
      case 4006: error_string="Invalid type or size of the array or corrupted dynamic array object";                                                                    break;
      case 4007: error_string="Not enough memory to reallocate the array or an attempt to change the dynamic array size";                                               break;
      case 4008: error_string="Not enough memory to reallocate the string";                                                                                             break;
      case 4009: error_string="Uninitialized string";                                                                                                                   break;
      case 4010: error_string="Invalid time and/or date value";                                                                                                         break;
      case 4011: error_string="Requested array size exceeds 2 GB";                                                                                                      break;
      case 4012: error_string="Incorrect pointer";                                                                                                                      break;
      case 4013: error_string="Incorrect pointer type";                                                                                                                 break;
      case 4014: error_string="System function cannot be called";                                                                                                       break;
      //-- Charts
      case 4101: error_string="Incorrect chart identifier";                                                                                                             break;
      case 4102: error_string="Chart not responding";                                                                                                                   break;
      case 4103: error_string="Chart not found";                                                                                                                        break;
      case 4104: error_string="No Expert Advisor on the chart to handle the event";                                                                                     break;
      case 4105: error_string="Chart opening error";                                                                                                                    break;
      case 4106: error_string="Error when changing chart symbol and period";                                                                                            break;
      case 4107: error_string="Incorrect timer value";                                                                                                                  break;
      case 4108: error_string="Error when creating the timer";                                                                                                          break;
      case 4109: error_string="Incorrect chart property identifier";                                                                                                    break;
      case 4110: error_string="Error when creating the screenshot";                                                                                                     break;
      case 4111: error_string="Chart navigation error";                                                                                                                 break;
      case 4112: error_string="Template application error";                                                                                                             break;
      case 4113: error_string="Subwindow with the specified indicator not found";                                                                                       break;
      case 4114: error_string="Error when adding indicator to the chart";                                                                                               break;
      case 4115: error_string="Error when removing indicator from the chart";                                                                                           break;
      case 4116: error_string="The indicator not found in the specified chart";                                                                                         break;
      //-- Graphical objects
      case 4201: error_string="Error when working with the graphical object";                                                                                           break;
      case 4202: error_string="The graphical object not found";                                                                                                         break;
      case 4203: error_string="Incorrect identifier of the graphical object property";                                                                                  break;
      case 4204: error_string="Unable to get the date corresponding to the value";                                                                                      break;
      case 4205: error_string="Unable to get the value corresponding to the date";                                                                                      break;
      //-- MarketInfo
      case 4301: error_string="Unknown symbol";                                                                                                                         break;
      case 4302: error_string="The symbol not selected in MarketWatch";                                                                                                 break;
      case 4303: error_string="Incorrect symbol property identifier";                                                                                                   break;
      case 4304: error_string="Unknown time of the last tick (no ticks)";                                                                                               break;
      //-- Access to history
      case 4401: error_string="Requested history not found!";                                                                                                           break;
      case 4402: error_string="Incorrect history property identifier";                                                                                                  break;
      //-- Global_Variables
      case 4501: error_string="Global variable of the client terminal not found";                                                                                       break;
      case 4502: error_string="Global variable of the client terminal with this name already exists";                                                                   break;
      case 4510: error_string="Failed to send the message";                                                                                                             break;
      case 4511: error_string="Failed to play the sound";                                                                                                               break;
      case 4512: error_string="Incorrect program property identifier";                                                                                                  break;
      case 4513: error_string="Incorrect terminal property identifier";                                                                                                 break;
      case 4514: error_string="Failed to export the file by ftp";                                                                                                       break;
      //-- Buffers of custom indicators
      case 4601: error_string="Not enough memory to allocate indicator buffers";                                                                                        break;
      case 4602: error_string="Incorrect index of the custom indicator buffer";                                                                                         break;
      //-- Custom indicator properties
      case 4603: error_string="Incorrect custom indicator property identifier";                                                                                         break;
      //-- Account
      case 4701: error_string="Incorrect account property identifier";                                                                                                  break;
      case 4751: error_string="Incorrect trading property identifier";                                                                                                  break;
      case 4752: error_string="The Expert Advisor is not allowed to trade";                                                                                             break;
      case 4753: error_string="The position not found";                                                                                                                 break;
      case 4754: error_string="The order not found";                                                                                                                    break;
      case 4755: error_string="The trade not found";                                                                                                                    break;
      case 4756: error_string="Failed to send the trade request";                                                                                                       break;
      //-- Indicators
      case 4801: error_string="Unknown symbol";                                                                                                                         break;
      case 4802: error_string="Unable to create the indicator";                                                                                                         break;
      case 4803: error_string="Not enough memory to add the indicator";                                                                                                 break;
      case 4804: error_string="Unable to apply the indicator to another indicator";                                                                                     break;
      case 4805: error_string="Error when adding the indicator";                                                                                                        break;
      case 4806: error_string="Requested data not found";                                                                                                               break;
      case 4807: error_string="Incorrect indicator handle";                                                                                                             break;
      case 4808: error_string="Invalid number of parameters when creating the indicator";                                                                               break;
      case 4810: error_string="Custom indicator name should be the first parameter in the array";                                                                       break;
      case 4809: error_string="No parameters to create the indicator";                                                                                                  break;
      case 4811: error_string="Invalid parameter type in the array when creating the indicator";                                                                        break;
      case 4812: error_string="Incorrect index of the requested indicator buffer";                                                                                      break;
      //-- Depth of market
      case 4901: error_string="Unable to add the depth of market";                                                                                                      break;
      case 4902: error_string="Unable to delete the depth of market";                                                                                                   break;
      case 4903: error_string="Unable to get data from the depth of market";                                                                                            break;
      case 4904: error_string="Error when subscribing to get new data from the depth of market";                                                                        break;
      //-- File operations
      case 5001: error_string="The number of files open at the same time cannot exceed 64";                                                                             break;
      case 5002: error_string="Invalid file name";                                                                                                                      break;
      case 5003: error_string="File name too long";                                                                                                                     break;
      case 5004: error_string="File opening error";                                                                                                                     break;
      case 5005: error_string="Not enough memory to cache read";                                                                                                        break;
      case 5006: error_string="File deleting error";                                                                                                                    break;
      case 5007: error_string="The file with this handle has already been closed or has never been opened";                                                             break;
      case 5008: error_string="Incorrect file handle";                                                                                                                  break;
      case 5009: error_string="The file must be open for writing";                                                                                                      break;
      case 5010: error_string="The file must be open for reading";                                                                                                      break;
      case 5011: error_string="The file must be open in binary mode";                                                                                                   break;
      case 5012: error_string="The file must be open in text mode";                                                                                                     break;
      case 5013: error_string="The file must be open in text mode or CSV format";                                                                                       break;
      case 5014: error_string="The file must be open in CSV format";                                                                                                    break;
      case 5015: error_string="File reading error";                                                                                                                     break;
      case 5016: error_string="String size must be specified for the file that is open in binary mode";                                                                 break;
      case 5017: error_string="There must be text file for string arrays and a binary file for all other arrays";                                                       break;
      case 5018: error_string="This is not a file, it is a directory";                                                                                                  break;
      case 5019: error_string="The file does not exist";                                                                                                                break;
      case 5020: error_string="The file cannot be rewritten";                                                                                                           break;
      case 5021: error_string="Incorrect directory name";                                                                                                               break;
      case 5022: error_string="The directory does not exist";                                                                                                           break;
      case 5023: error_string="This is not a directory, it is a file";                                                                                                  break;
      case 5024: error_string="The directory cannot be deleted";                                                                                                        break;
      case 5025: error_string="Failed to clear the directory (can happen if one or more files is blocked and the deletion was not successful)";                         break;
      //-- String formatting
      case 5030: error_string="No date in the string";                                                                                                                  break;
      case 5031: error_string="Incorrect date in the string";                                                                                                           break;
      case 5032: error_string="Incorrect time in the string";                                                                                                           break;
      case 5033: error_string="Error converting string to date";                                                                                                        break;
      case 5034: error_string="Not enough memory for the string";                                                                                                       break;
      case 5035: error_string="String length is less than expected";                                                                                                    break;
      case 5036: error_string="Number too large, bigger than ULONG_MAX";                                                                                                break;
      case 5037: error_string="Incorrect format string";                                                                                                                break;
      case 5038: error_string="The number of format specifiers is bigger than the number of parameters";                                                                break;
      case 5039: error_string="The number of parameters is bigger than the number of format specifiers";                                                                break;
      case 5040: error_string="Corrupted string type parameter";                                                                                                        break;
      case 5041: error_string="Position outside of the string";                                                                                                         break;
      case 5042: error_string="0 added to the end of the string, content-free operation";                                                                               break;
      case 5043: error_string="Unknown data type when converting to string";                                                                                            break;
      case 5044: error_string="Corrupted string object";                                                                                                                break;
      //-- Operations with arrays
      case 5050: error_string="Cannot copy incompatible arrays. String array can only be copied to another string array and numeric array to another numeric array";    break;
      case 5051: error_string="The receiving array is declared as AS_SERIES and its size is not sufficient";                                                            break;
      case 5052: error_string="Array too small, the starting position is outside of the array";                                                                         break;
      case 5053: error_string="Zero length array";                                                                                                                      break;
      case 5054: error_string="The array must be numeric";                                                                                                              break;
      case 5055: error_string="The array must be one-dimensional";                                                                                                      break;
      case 5056: error_string="The time series cannot be used";                                                                                                         break;
      case 5057: error_string="The array must be of the double type";                                                                                                   break;
      case 5058: error_string="The array must be of the float type";                                                                                                    break;
      case 5059: error_string="The array must be of the long type";                                                                                                     break;
      case 5060: error_string="The array must be of the int type";                                                                                                      break;
      case 5061: error_string="The array must be of the short type";                                                                                                    break;
      case 5062: error_string="The array must be of the char type";                                                                                                     break;
      //-- User errors

      default: error_string="The error is undefined";
     }
//---
   return(error_string);
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initialize the new bar
   CheckNewBar();
//--- Get position properties and update the values on the panel
   GetPositionProperties();
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- Print the deinitialization reason to the journal
   Print(GetDeinitReasonText(reason));
//--- When deleting from the chart
   if(reason==REASON_REMOVE)
      //--- Delete all objects relating to the info panel from the chart
      DeleteInfoPanel();
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- If the bar is not new, exit
   if(!CheckNewBar())
      return;
//--- If there is a new bar
   else
     {
      GetBarsData();  // Get bar data
      TradingBlock(); // Check the conditions and trade
     }
//--- Get the properties and update the values on the panel
   GetPositionProperties();
  }
//+------------------------------------------------------------------+
//| TRADE EVENT                                                      |
//+------------------------------------------------------------------+
void OnTrade()
  {
//--- Get position properties and update the values on the panel
   GetPositionProperties();
  }
//+------------------------------------------------------------------+
